package no.edu.esign;

import com.fasterxml.jackson.databind.ObjectMapper;
import io.swagger.v3.oas.annotations.enums.SecuritySchemeIn;
import io.swagger.v3.oas.annotations.enums.SecuritySchemeType;
import io.swagger.v3.oas.annotations.security.SecurityScheme;
import io.swagger.v3.oas.models.ExternalDocumentation;
import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.oas.models.info.Info;
import io.swagger.v3.oas.models.security.SecurityRequirement;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;

import java.text.SimpleDateFormat;
import java.util.List;

@SecurityScheme(
        name = "Authorization",
        type = SecuritySchemeType.APIKEY,
        in = SecuritySchemeIn.HEADER
)
@EnableWebSecurity
@Configuration
public class SignContext  {

    @Bean
    public MappingJackson2HttpMessageConverter mappingJackson2HttpMessageConverter() {
        ObjectMapper mapper = new ObjectMapper();
        mapper.setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"));
        return new MappingJackson2HttpMessageConverter(mapper);
    }

    @Bean
    public OpenAPI eSigneringOpenAPI() {
        return new OpenAPI()
                .info(new Info().title("eSignering - API")
                        .description("Api for å generere signerignsoppdrag og hente signertedokumenter")
                        .version("v0.0.1"))
                .security(List.of(new SecurityRequirement().addList("Authorization")))
                .externalDocs(new ExternalDocumentation()
                        .description("Postens signerings løsning")
                        .url("https://signering-docs.readthedocs.io"));
    }

}
