package no.edu.esign;

import no.edu.esign.model.identity.User;
import no.edu.esign.model.identity.UserDetailsImpl;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

@Service
public class UserDetailsServiceImpl implements UserDetailsService {
    private static final Logger logger = LoggerFactory.getLogger(UserDetailsServiceImpl.class);
    @SuppressWarnings("FieldCanBeLocal")
    private final String CLIENT_SECRET= "jwt.client.%s.secret"; // will be in form jwt.client.{client_id}.secret
    @SuppressWarnings("FieldCanBeLocal")
    private final String CLIENT_SCOPE= "jwt.client.%s.scope"; // will be in form jwt.client.{client_id}.scope
    @Autowired
    Environment env;

    @Override
    public UserDetails loadUserByUsername(String client_id) throws UsernameNotFoundException {
        String secret = loadFromConfig(CLIENT_SECRET, client_id, "secret");
        String scope = loadFromConfig(CLIENT_SCOPE, client_id, "scope");

        String bCrypt = new BCryptPasswordEncoder().encode(secret);
        User user = new User(client_id, bCrypt, scope);
        return UserDetailsImpl.build(user);
    }

    private String loadFromConfig(String propKey, String client_id, String field) {
        String aProp = env.getProperty(String.format(propKey, client_id));

        if (StringUtils.isBlank(aProp)){
            logger.info("Invalid config! Could not find client {} for client_id: {}", field, client_id);
            throw new IllegalArgumentException("Invalid client_id or secret, client_id: "+ client_id);
        }
        return aProp;
    }
}

