package no.edu.esign.controller;


import io.swagger.v3.oas.annotations.tags.Tag;
import no.edu.esign.JwtService;
import no.edu.esign.model.identity.UserDetailsImpl;
import no.edu.esign.model.identity.AccessTokenRequest;
import no.edu.esign.model.identity.AccessTokenResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.ws.rs.Consumes;
import javax.ws.rs.Produces;

@Tag(name = "Access token API", description = "Use this api to create access token (not needed when accessing via Gravitee)")
@RestController
@RequestMapping("/api/identity")
@Produces({"application/json"})
@Consumes({"application/json"})
public class IdentityController {
    private static final String TOKEN_TYPE_BEARER = "Bearer";

    Logger logger = LoggerFactory.getLogger(IdentityController.class);
    @Autowired
    private JwtService jwtService;
    @Autowired
    private AuthenticationManager authenticationManager;

    @PostMapping(path = "/token", produces = MediaType.APPLICATION_JSON_VALUE,
            consumes = {MediaType.APPLICATION_JSON_VALUE, MediaType.APPLICATION_FORM_URLENCODED_VALUE})
    public @Valid @ResponseBody
    ResponseEntity<Object> createAccessToken(@Valid @RequestBody AccessTokenRequest request) {
        logger.info("Creating access token for client: " + request.getClient());

        Authentication authentication = authenticationManager.authenticate(new UsernamePasswordAuthenticationToken(request.getClient(), request.getSecret()));
        SecurityContextHolder.getContext().setAuthentication(authentication);

        UserDetailsImpl userDetails = (UserDetailsImpl) authentication.getPrincipal();
        String jwt = jwtService.generateJwtToken(userDetails);
        AccessTokenResponse accessTokenResp = AccessTokenResponse.builder()
                .accessToken(jwt)
                .expiresInMillis(jwtService.getJwtExpiryMillis())
                .tokenType(TOKEN_TYPE_BEARER)
                .build();
        return ResponseEntity.ok(accessTokenResp);
    }
}
