package no.edu.esign.model.sign;


import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Builder;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

import javax.validation.constraints.*;
import java.util.List;

@Getter
@Setter
@ToString(exclude = {"signersNin", "title", "description", "document"})
@Builder
public class SignRequest {
    @NotNull
    @JsonProperty("JobType")
    private JobType jobType;
    @NotNull
    @JsonProperty("SignatureType")
    private SignatureType signatureType;
    @Min(value = 3)
    @Max(value = 4)
    @JsonProperty("SecurityLevel")
    private int securityLevel;
    @NotBlank
    @JsonProperty("Description")
    private String description;
    @NotBlank
    @JsonProperty("Title")
    private String title;
    @NotBlank
    @JsonProperty("Document")
    private String document; // base64 encoded String
    @JsonProperty("SignersNin")
    @NotEmpty
    @Size(min = 1, max = 10)
    private List<String> signersNin;
    @NotNull
    @JsonProperty("ExistUrls")
    private ExitUrls existUrls;

    // used by jackson
    public SignRequest() {}

    public SignRequest(JobType JobType, SignatureType SignatureType, int securityLevel, String description, String title, String document, List<String> signersNin, ExitUrls existUrls) {
        this.jobType = JobType;
        this.signatureType = SignatureType;
        this.securityLevel = securityLevel;
        this.description = description;
        this.title = title;
        this.document = document;
        this.signersNin = signersNin;
        this.existUrls = existUrls;
    }

}



