package no.edu.esign.util;

import no.edu.esign.model.sign.DocumentRequest;
import no.edu.esign.model.sign.SignRequest;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.StringUtils;

import java.net.MalformedURLException;
import java.net.URL;

public class AssertionsUtil {

    public static void assertSignRequest(SignRequest request) {
        request.getSignersNin().forEach(nin -> {
            if (nin.length() != 11) throw new IllegalArgumentException("Invalid nin: "+ nin);
            if (!nin.matches("\\d{11}")) throw new IllegalArgumentException("11 nr required, Invalid nin: "+ nin);
        } );

        validateUrl(request.getExistUrls().getCompletion());
        validateUrl(request.getExistUrls().getRejection());
        validateUrl(request.getExistUrls().getError());

        if (!Base64.isBase64(request.getDocument())){
            throw new IllegalArgumentException("Invalid document to sign, it should be base64 encoded");
        }
    }

    public static void validateUrl(String url) throws IllegalArgumentException {
        try {
            URL uri = new URL(url);
        } catch (MalformedURLException e) {
            throw new IllegalArgumentException("Invalid url: "+ url);
        }
    }

    public static void assertDocumentRequest(DocumentRequest request) {
        if (request == null){
            throw new IllegalArgumentException("Empty request");
        }
        if (StringUtils.isBlank(request.getXadesUrl()) && StringUtils.isBlank(request.getPadesUrl())){
            throw new IllegalArgumentException("Both XadesUrl and PadesUrl can't be empty");
        }
    }
}
